import { aql } from "arangojs";
import type { Document } from "arangojs/documents";
import type { IncomingMessage, ServerResponse } from "http";
import { authorizeRequest } from "../../lib/butil";
import { Permissions } from "../../lib/permissions";
import { db } from "../../server";
import type { Graph } from "./_types";

export async function get(req: IncomingMessage, res: ServerResponse) {
	const [_, exit] = await authorizeRequest(req, res, db, [Permissions.VIEW]);
	if (exit) return;

	let graphs: (Graph & { graph: string })[] = [];
	const cursor = await db.query(aql`
		FOR doc IN config
			FILTER doc.type == "graph"
			RETURN doc
	`);
	while (cursor.hasNext) {
		const next: Document<Graph> = await cursor.next();
		graphs.push({
			graph: next._key,
			type: next.type,
			data: next.data
		});
	}
	res.writeHead(200);
	res.end(JSON.stringify(graphs));
}
